/*
-------------------------------------------------------------------------------
                                I   M   L   I   B
				       1.1
-------------------------------------------------------------------------------

This software is Copyright (C) 1998 By The Rasterman (Carsten Haitzler). I
accept no responsability for anythign this software may or may not do to
your system - you use it completely at your own risk. This software comes
under the LGPL and GPL licences. The library itself is LGPL (all software in
Imlib and gdk_imlib directories) see the COPYING and COPYING.LIB files for
full legal details.
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xos.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/shape.h>
#include <gdk/gdk.h>
#include <gdk/gdkprivate.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <time.h>
#include <netinet/in.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <sys/types.h>

#ifdef _HAVE_STRING_H
#include <string.h>
#elif _HAVE_STRINGS_H
#include <strings.h>
#endif

#ifndef SYSTEM_IMRC
#define SYSTEM_IMRC "/etc/imrc"
#endif /* endef SYSTEM_IMRC */

typedef struct _GdkImlibBorder
{
   gint left,right;
   gint top,bottom;
} GdkImlibBorder;

typedef struct _GdkImlibColor
{
   gint r,g,b;
   gint pixel;
} GdkImlibColor;

typedef struct _GdkImlibColorModifier
{
   gint gamma;
   gint brightness;
   gint contrast;
} GdkImlibColorModifier;

typedef struct _GdkImlibImage
{
   gint rgb_width,       rgb_height;
   unsigned char         *rgb_data;
   unsigned char         *alpha_data;
   gchar                 *filename;
/* the below information is private */
   gint                  width,height;
   GdkImlibColor         shape_color;
   GdkImlibBorder        border;
   GdkPixmap             *pixmap;
   GdkBitmap             *shape_mask;
   gchar                  cache;
   GdkImlibColorModifier mod,rmod,gmod,bmod;
   unsigned char         rmap[256],gmap[256],bmap[256];
} GdkImlibImage;

struct image_cache
{
   gchar              *file;
   GdkImlibImage      *im;
   gint               refnum;
   gchar              dirty;
   struct image_cache *prev;
   struct image_cache *next;
};

struct pixmap_cache
{
   GdkImlibImage       *im;
   gchar               *file;
   gchar               dirty;
   gint                width,height;
   GdkPixmap           *pmap;
   GdkBitmap           *shape_mask;
   gint                refnum;
   struct pixmap_cache *prev;
   struct pixmap_cache *next;
};

typedef struct _xdata
{
   Display         *disp; 
   gint            screen; 
   Window          root; 
   Visual          *visual; 
   gint            depth; 
   Colormap        root_cmap; 
   gchar           shm; 
   gchar           shmp; 
   gint            shm_event; 
   XImage          *last_xim;
   XImage          *last_sxim;
   XShmSegmentInfo last_shminfo;
   XShmSegmentInfo last_sshminfo;
   Window          base_window;
   GdkWindow       *gdk_win;
   GdkColormap     *gdk_cmap;
} Xdata;

typedef struct _imlibdata
{
   gint          num_colors; 
   GdkImlibColor palette[256]; 
   GdkImlibColor palette_orig[256]; 
   gint          fast_rgb[32][32][32]; 
   gint          fast_err[32][32][32]; 
   gint          fast_erg[32][32][32];
   gint          fast_erb[32][32][32];
   gint          render_type; 
   gint          max_shm;
   Xdata         x; 
   gint          byte_order;
   struct _cache 
     {
	gchar    on_image;
	gint     size_image;
	gint     num_image;
	gint     used_image;
	struct   image_cache *image;
	gchar    on_pixmap;
	gint     size_pixmap;
	gint     num_pixmap;
	gint     used_pixmap;
	struct   pixmap_cache *pixmap;
     } cache;
   gchar                 fastrend; 
   gchar                 hiq; 
   GdkImlibColorModifier mod,rmod,gmod,bmod;
   unsigned char         rmap[256],gmap[256],bmap[256];
   gchar                 fallback;
} ImlibData;

#define RT_PLAIN_PALETTE       0
#define RT_PLAIN_PALETTE_FAST  1
#define RT_DITHER_PALETTE      2
#define RT_DITHER_PALETTE_FAST 3
#define RT_PLAIN_TRUECOL       4
/* a special high-quality renderer for people with 15 and 16bpp that dithers */
#define RT_DITHER_TRUECOL      5
